#include "HelloWorldScene.h"
#include "WdTableView.h"

//USING_NS_CC;

HelloWorld::HelloWorld()
: m_table(NULL)
, m_data(NULL)
{
    
}
HelloWorld::~HelloWorld() {
    CC_SAFE_RELEASE(m_table);
    CC_SAFE_RELEASE(m_data);
}

CCScene* HelloWorld::scene()
{
    // 'scene' is an autorelease object
    CCScene *scene = CCScene::create();
    
    // 'layer' is an autorelease object
    HelloWorld *layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    //////////////////////////////
    // 1. super init first
    if ( !CCLayer::init() )
    {
        return false;
    }
    
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    CCPoint origin = CCDirector::sharedDirector()->getVisibleOrigin();

    /////////////////////////////
    // 2. add a menu item with "X" image, which is clicked to quit the program
    //    you may modify it.

    // add a "close" icon to exit the progress. it's an autorelease object
    CCMenuItemImage *pCloseItem = CCMenuItemImage::create(
                                        "CloseNormal.png",
                                        "CloseSelected.png",
                                        this,
                                        menu_selector(HelloWorld::menuCloseCallback));
    
	pCloseItem->setPosition(ccp(origin.x + visibleSize.width - pCloseItem->getContentSize().width/2 ,
                                origin.y + pCloseItem->getContentSize().height/2));

    // create menu, it's an autorelease object
    CCMenu* pMenu = CCMenu::create(pCloseItem, NULL);
    pMenu->setPosition(CCPointZero);
    this->addChild(pMenu, 1);

    /////////////////////////////
    // 3. add your codes below...

    // add a label shows "Hello World"
    // create and initialize a label
    
    CCLabelTTF* pLabel = CCLabelTTF::create("Hello World", "Arial", 24);
    
    // position the label on the center of the screen
    pLabel->setPosition(ccp(origin.x + visibleSize.width/2,
                            origin.y + visibleSize.height - pLabel->getContentSize().height));

    // add the label as a child to this layer
    this->addChild(pLabel, 1);

    // add "HelloWorld" splash screen"
    CCSprite* pSprite = CCSprite::create("HelloWorld.png");

    // position the sprite on the center of the screen
    pSprite->setPosition(ccp(visibleSize.width/2 + origin.x, visibleSize.height/2 + origin.y));

    // add the sprite as a child to this layer
    this->addChild(pSprite, 0);
    
    /// 测试table
    /// 添加测试用的数据
    initData();
    addTableView();
    
    return true;
}


void HelloWorld::menuCloseCallback(CCObject* pSender)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT) || (CC_TARGET_PLATFORM == CC_PLATFORM_WP8)
	CCMessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
#else
    CCDirector::sharedDirector()->end();
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
#endif
}

#pragma mark - table
void HelloWorld::initData() {
    do{
        CC_BREAK_IF(get_data());
        set_data(CCArray::create());
        for (int index = 0; index < 7; ++index) {
            CCString* labelValue    = CCString::createWithFormat("Cell : %d", index);
            get_data()->addObject(labelValue);
        }
        break;
    }while(false);
}
void HelloWorld::addTableView()  {
    CCSize winsize = CCDirector::sharedDirector()->getWinSize();
    /// 创建一个table
    set_table(WdTableView::create(this, CCSize(400.0f, 500.0f)));
    this->addChild(get_table(), 2);
    get_table()->setDelegate(this);
    /// 这个table只支持从上到下
    get_table()->setVerticalFillOrder(kCCTableViewFillTopDown);
    get_table()->setPosition(ccp(winsize.width / 2-200, winsize.height / 2-250));
    
    /// 添加动画事件的操作
    CCMenuItemLabel* add    = CCMenuItemLabel::create(CCLabelTTF::create("Add Cell", "Arial", 28), this, menu_selector(HelloWorld::addCell));
    CCMenuItemLabel* del    = CCMenuItemLabel::create(CCLabelTTF::create("Del Cell", "Arial", 28), this, menu_selector(HelloWorld::deleteCell));
    add->setPositionY(100);
    del->setPositionY(-100);
    CCMenu* menu    = CCMenu::create(add, del, NULL);
    addChild(menu, 4);
    
}
void HelloWorld::addCell(cocos2d::CCObject *pSender) {
    WdTableView* table  = static_cast<WdTableView*>(get_table());
    /// 切记要更新data
    get_data()->insertObject(CCString::create("Cell added"), 5);
    /// 点击之后，1秒做动画
    table->insertCellAtIndex(5, 1.0f);
    /// 注意这两个方法的调用顺序
}
void HelloWorld::deleteCell(cocos2d::CCObject *pSender) {
    WdTableView* table  = static_cast<WdTableView*>(get_table());
    /// 切记要更新data
    get_data()->removeObjectAtIndex(5);
    /// 点击之后，1秒做动画
    table->removeCellAtIndex(5);
}

#pragma mark - table delegate
CCSize HelloWorld::cellSizeForTable(CCTableView *table) {
    return CCSize(400.0f, 80.0f);
}
CCTableViewCell* HelloWorld::tableCellAtIndex(CCTableView *table, unsigned int idx) {
    CCTableViewCell* cell   = table->dequeueCell();
    if (!cell) {
        cell    = new CCTableViewCell();
        cell->autorelease();
    }
    cell->removeAllChildren();
    CCString* str   = static_cast<CCString*>(get_data()->objectAtIndex(idx));
    CCLabelTTF* label   = CCLabelTTF::create(str->getCString(), "Arial", 28.0f);
    label   ->setPosition(ccp(200, 40));
    cell->addChild(label, 3);
    /// 添加背景色，区别视觉
    ccColor4B col   = ccc4(128, 128, 128, 255);
    if (idx % 2) {
        col =    ccc4(200, 128, 225, 255);
    }
    CCLayerColor* color = CCLayerColor::create(col, 400, 80);
    cell->addChild(color, 1);
    return cell;
}
unsigned int HelloWorld::numberOfCellsInTableView(CCTableView *table) {
    int number  = get_data()->count();
    return number;
}
void HelloWorld::tableCellTouched(CCTableView* table, CCTableViewCell* cell) {
    
}
void HelloWorld::scrollViewDidScroll(CCScrollView* view) {
    
}
void HelloWorld::scrollViewDidZoom(CCScrollView* view) {
    
}