//
//  WdTableView.h
//  tttt
//
//  Created by Windy on 12/27/13.
//
//

#ifndef __tttt__WdTableView__
#define __tttt__WdTableView__

#include "cocos2d.h"
#include "cocos-ext.h"

class WdTableView
: public cocos2d::extension::CCTableView, /** 继承cctableview */
public cocos2d::CCRGBAProtocol /** 继承rgb通道, 使该控件能够做渐隐和渐显 */
{
    /// @{
    /// @name 创建方法
public:
    WdTableView();
    virtual ~WdTableView();
    /**
     *    @brief    根据数据代理和大小去创建一个table
     *
     *    @Modified by chenbaicai at 2013-12-27 18:34:02
     *    @param     dataSource     数据代理
     *    @param     size     控件大小
     *
     *    @return    创建好的table大小
     **/
    static WdTableView* create(cocos2d::extension::CCTableViewDataSource* dataSource, cocos2d::CCSize size);
    
    /**
     *    @brief    根据数据代理的大小，主要容器去创建一个table
     *
     *    @Modified by chenbaicai at 2013-12-27 18:36:46
     *    @param     dataSource     数据代理
     *    @param     size     控件大小
     *    @param     container     主要容器（如果自己设置，本类不负责retain）
     *
     *    @return    返回一个创建成功的table
     **/
    static WdTableView* create(cocos2d::extension::CCTableViewDataSource* dataSource, cocos2d::CCSize size, cocos2d::CCNode *container);
    
    /// @} end 创建方法
    /// @{
    /// @name 属性方法
    /**
     *    @brief    动态插入cell
     *
     *    @Modified by chenbaicai at 2013-12-27 18:42:21
     *    @param     idx     下标位置
     *    @param     insertDelayTime     这个delaytime只是为了跟其他动画配合，可以独立一个函数
     **/
    void insertCellAtIndex(unsigned int idx, float insertDelayTime);
    /**
     *    @brief    某个点下cell下标
     *
     *    @Modified by chenbaicai at 2013-12-27 18:46:40
     *    @param     x     x
     *    @param     y     y
     *
     *    @return    下标index
     **/
    int indexFromPoint(int x, int y);
    
    /**
     *    @brief    返回指定下标的位置
     *
     *    @Modified by chenbaicai at 2013-12-27 18:47:17
     *    @param     index     下标
     *
     *    @return    坐标
     **/
    cocos2d::CCPoint pointFromIndex(int index);
    
    /**
     *    @brief    把cell放置到指定的index下
     *
     *    @Modified by chenbaicai at 2013-12-27 18:47:46
     *    @param     index     要移动到的目标的下标
     *    @param     cell     当前要移动的cell
     *    @param     animate     是否要作动画
     **/
    void _setIndexForCell(unsigned int index, cocos2d::extension::CCTableViewCell *cell, bool animate);
    
    /**
     *    @brief    移除指定下标下的cell
     *
     *    @Modified by chenbaicai at 2013-12-27 18:50:36
     *    @param     idx     要移除的cell的下标
     **/
    void removeCellAtIndex(unsigned int idx);
    
    /**
     *    @brief    作推迟插入cell的动画时，回调插入cell的方法
     *
     *    @Modified by chenbaicai at 2013-12-27 18:52:04
     **/
    void delayInsertCell();
    
    
    /// @note   这些都是rgb通道的方法
    /**
     * Changes the color with R,G,B bytes
     *
     * @param color Example: ccc3(255,100,0) means R=255, G=100, B=0
     */
    virtual void setColor(const cocos2d::ccColor3B& color);
    
    /**
     * Returns color that is currently used.
     *
     * @return The ccColor3B contains R,G,B bytes.
     */
    virtual const cocos2d::ccColor3B& getColor(void);
    
    /**
     * Returns the displayed color.
     *
     * @return The ccColor3B contains R,G,B bytes.
     */
    virtual const cocos2d::ccColor3B& getDisplayedColor(void);
    
    /**
     * Returns the displayed opacity.
     *
     * @return  The opacity of sprite, from 0 ~ 255
     */
    virtual GLubyte getDisplayedOpacity(void);
    /**
     * Returns the opacity.
     *
     * The opacity which indicates how transparent or opaque this node is.
     * 0 indicates fully transparent and 255 is fully opaque.
     *
     * @return  The opacity of sprite, from 0 ~ 255
     */
    virtual GLubyte getOpacity(void);
    
    /**
     * Changes the opacity.
     *
     * @param   value   Goes from 0 to 255, where 255 means fully opaque and 0 means fully transparent.
     */
    virtual void setOpacity(GLubyte opacity);
    
    // optional
    /**
     * Changes the OpacityModifyRGB property.
     * If thie property is set to true, then the rendered color will be affected by opacity.
     * Normally, r = r * opacity/255, g = g * opacity/255, b = b * opacity/255.
     *
     * @param   bValue  true then the opacity will be applied as: glColor(R,G,B,opacity);
     *                  false then the opacity will be applied as: glColor(opacity, opacity, opacity, opacity);
     */
    virtual void setOpacityModifyRGB(bool bValue);
    /**
     * Returns whether or not the opacity will be applied using glColor(R,G,B,opacity)
     * or glColor(opacity, opacity, opacity, opacity)
     *
     * @return  Returns opacity modify flag.
     */
    virtual bool isOpacityModifyRGB(void);
    /**
     *  whether or not color should be propagated to its children.
     */
    virtual bool isCascadeColorEnabled(void);
    virtual void setCascadeColorEnabled(bool cascadeColorEnabled);
    
    /**
     *  recursive method that updates display color
     */
    virtual void updateDisplayedColor(const cocos2d::ccColor3B& color);
    
    /**
     *  whether or not opacity should be propagated to its children.
     */
    virtual bool isCascadeOpacityEnabled(void);
    virtual void setCascadeOpacityEnabled(bool cascadeOpacityEnabled);
    
    /**
     *  recursive method that updates the displayed opacity.
     */
    virtual void updateDisplayedOpacity(GLubyte opacity);
    /// @} end 属性方法
    
    /// @{
    /// @name 属性
public:
    int m_indexToInsert;                // 延时插入新数据时备份
protected:
    GLubyte        _displayedOpacity;      /// 展示的透明度
    GLubyte     _realOpacity;           /// 真实透明度
    cocos2d::ccColor3B    _displayedColor;/// 真实色彩
    cocos2d::ccColor3B   _realColor;    /// 展示色彩
    bool        _cascadeColorEnabled;   /// 是否开启色彩变幻
    bool        _cascadeOpacityEnabled; /// 是否开启透明渐变
    /// @} end 属性
};
#endif /* defined(__tttt__WdTableView__) */